from ...config import get_config
from ...token import get_access_token
from ...common import with_types

import requests

API_VERSION = "v1"
SERVICE_NAME = "rci"


@with_types(str, str, dict)
def execute(endpoint_id, command_type, command):
    r"""
    Invokes command on the endpoint and returns its result in a response. Body can be any valid json that is meaningful
     for the target endpoint.

    :param endpoint_id: Unique endpoint identifier.
    :param command_type: Represents the type of the command client wants to invoke on the endpoint.
     Command type MUST be a non-empty alpha-numeric string identifying the command type to the endpoint.
    :param command: json-formatted command
    :return: command invocation result
    """
    host = get_config()['host']
    token = get_access_token()
    headers = {
        "Authorization": f"Bearer {token}",
        "Content-Type": "application/json"
    }

    resp = requests.post(
        f'https://{host}/{SERVICE_NAME}/api/{API_VERSION}/endpoints/{endpoint_id}/commands/{command_type}',
        headers=headers,
        json=command)
    resp.raise_for_status()

    return resp.json()



from functools import wraps


def with_types(*types):
    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            for tp, arg in zip(types, args):
                if not isinstance(arg, tp):
                    raise TypeError
            return fn(*args, **kwargs)
        return wrapper
    return decorator



from ...config import get_config
from ...token import get_access_token
from ...common import with_types
from datetime import datetime

import requests

API_VERSION = "v1"
SERVICE_NAME = "epts"

_time_series_config = None


@with_types(list)
def get_time_series_config(application_names=None):
    r"""
    Returns configurations of all time series.

    :param application_names: Application name, one or multiple. If not specified,
     the data is returned for all available applications.
    :return: json with time series configuration
    """
    if not _time_series_config:
        _set_ts_config(application_names)

    return _time_series_config


@with_types(list, datetime, datetime, list, str)
def get_time_series(time_series_names, from_date, to_date, endpoint_id=None, sort=None):
    r"""
    Returns time series data points within the specified time range ordered by timestamp and grouped by endpoints.

    :param time_series_names: One or more time series names.
    :param from_date: Start date to retrieve data points.
    :param to_date: End date to retrieve data points.
    :param endpoint_id: One or more endpoint IDs. If not specified, data is returned for all available endpoints.
    :param sort: Sorting order by timestamp. (one of 'ASC', 'DESC' - default: 'ASC')
    :return: json with time-series data
    """
    host = get_config()['host']
    token = get_access_token()
    headers = {
        "Authorization": f"Bearer {token}"
    }
    params = {
        "timeSeriesName": time_series_names,
        "endpointId": endpoint_id,
        "fromDate": from_date.isoformat(),
        "toDate": to_date.isoformat(),
        "sort": sort
    }

    resp = requests.get(f'https://{host}/{SERVICE_NAME}/api/{API_VERSION}/time-series/data',
                        params=params, headers=headers)
    resp.raise_for_status()

    return resp.json()


@with_types(list, list)
def get_last_time_series(time_series_names, endpoint_id=None):
    r"""
    Returns the most recent data points for time series for endpoint(s).

    :param time_series_names: Time series names, one or multiple.
    :param endpoint_id: Endpoint ID, one or multiple. If not specified, the data is returned for all available endpoints.
    :return: json with the most recent time-series data
    """
    host = get_config()['host']
    token = get_access_token()
    headers = {
        "Authorization": f"Bearer {token}"
    }
    params = {
        "timeSeriesName": time_series_names,
        "endpointId": endpoint_id
    }

    resp = requests.get(f'https://{host}/{SERVICE_NAME}/api/{API_VERSION}/time-series/last',
                        params=params, headers=headers)
    resp.raise_for_status()

    return resp.json()


def _set_ts_config(application_names=None):
    global _time_series_config

    host = get_config()['host']
    token = get_access_token()
    headers = {
        "Authorization": f"Bearer {token}"
    }
    params = {
        "applicationNames": application_names
    }
    resp = requests.get(f'https://{host}/{SERVICE_NAME}/api/{API_VERSION}/time-series/config',
                        params=params, headers=headers)
    resp.raise_for_status()

    _time_series_config = resp.json()



import os
import json

from .file_utils import (is_kaa_dir_permissions_valid, KAA_CREDENTIALS_FILE, KAA_DIR, KAA_DEFAULT_CREDENTIALS,
                         KAA_CONFIG_FILE, KAA_DEFAULT_CONFIG)
from .exceptions import UnauthorizedFileAccessException
from .common import with_types


@with_types(str, str, str, str)
def set_credentials_file(uname=None, password=None, client_id=None, client_secret=None):
    r"""
    Creates or updates ~/.kaa/.credentials file on your local machine with given info.

    :param uname: kaa platform username
    :param password: kaa platform password
    :param client_id: your clientId
    :param client_secret: your client secret
    """
    if not is_kaa_dir_permissions_valid():
        raise UnauthorizedFileAccessException(KAA_DIR)

    creds = get_credentials()
    creds['username'] = uname
    creds['password'] = password
    creds['client_id'] = client_id
    creds['client_secret'] = client_secret

    with open(KAA_CREDENTIALS_FILE, 'w') as f:
        json.dump(creds, f)


def get_credentials():
    if os.path.isfile(KAA_CREDENTIALS_FILE):
        with open(KAA_CREDENTIALS_FILE) as f:
            return json.load(f)
    else:
        return KAA_DEFAULT_CREDENTIALS


@with_types(str, str, str)
def set_config_file(host=None, auth_host=None, realm=None):
    r"""
    Creates or updates ~/.kaa/.config file on your local machine with given info.

    :param host: host of kaa-cluster
    :param auth_host: keycloak host
    :param realm: realm
    """
    if not is_kaa_dir_permissions_valid():
        raise UnauthorizedFileAccessException(KAA_DIR)

    configs = get_config()
    configs['host'] = host
    configs['realm'] = realm
    configs['auth_host'] = auth_host

    with open(KAA_CONFIG_FILE, 'w') as f:
        json.dump(configs, f)


def get_config():
    if os.path.isfile(KAA_CONFIG_FILE):
        with open(KAA_CONFIG_FILE) as f:
            return json.load(f)
    else:
        return KAA_DEFAULT_CONFIG



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='kaa_sdk',
      version='0.1.3',
      description='Python client for Kaa API',
      author='Alexandr Agitolyev',
      author_email='aagitolyev@kaaiot.io',
      packages=find_packages(),
      install_requires=["requests"]
      )



import time

import requests
from .config import get_config, get_credentials


class Token:

    def __init__(self, access_token, expires_in, refresh_token, refresh_expires_in, token_type, scope):
        self._access_token = access_token
        self._expires_in = expires_in
        self._refresh_token = refresh_token
        self._refresh_expires_in = refresh_expires_in
        self._token_type = token_type
        self._scope = scope
        self._creation_time = time.time()

    @classmethod
    def of_json(cls, json_repr):
        return cls(json_repr['access_token'], json_repr['expires_in'], json_repr['refresh_token'],
                   json_repr['refresh_expires_in'], json_repr['token_type'], json_repr['scope'])

    def is_expired(self):
        return time.time() - self._creation_time > self._expires_in

    @property
    def access_token(self):
        return self._access_token


_token = None


def get_access_token():
    global _token
    headers = {
        "Accept": "application/json",
        "Content-Type": "application/x-www-form-urlencoded",
    }

    form_params = get_credentials()
    if not _token or _token.is_expired():
        resp = requests.post(_form_auth_url(), headers=headers, data=form_params)
        resp.raise_for_status()
        _token = Token.of_json(resp.json())

    return _token.access_token


def _form_auth_url():
    config = get_config()
    return f'https://{config["auth_host"]}/auth/realms/{config["realm"]}/protocol/openid-connect/token'



class UnauthorizedFileAccessException(BaseException):
    MESSAGE_PATTERN = 'You are not authorized to access {}'

    def __init__(self, file_path):
        global MESSAGE_PATTERN
        BaseException.__init__(self, MESSAGE_PATTERN.format(file_path))



import os

KAA_DIR = os.getenv('KAA_DIR', os.path.join(os.path.expanduser('~'), '.kaa'))
KAA_CREDENTIALS_FILE = os.path.join(KAA_DIR, '.credentials')
KAA_CONFIG_FILE = os.path.join(KAA_DIR, '.config')
TEST_FILE = os.path.join(KAA_DIR, '.test_file')


KAA_DEFAULT_CREDENTIALS = {
    "username": ' ',
    "password": ' ',
    "client_id": ' ',
    "client_secret": ' ',
    "grant_type": "password"
}

KAA_DEFAULT_CONFIG = {
    "auth_host": ' ',
    "host": ' ',
    "realm": ' '
}

_dir_permissions = None


def is_kaa_dir_permissions_valid():
    global _dir_permissions

    if not _dir_permissions:
        _dir_permissions = _permissions()

    return _dir_permissions


def _permissions():
    try:
        if not os.path.exists(KAA_DIR):
            os.mkdir(KAA_DIR)
        with open(TEST_FILE, 'w') as f:
            f.write('Just some text')
        os.remove(TEST_FILE)
        return True
    except:
        return False

